=== modified file 'NEWS'
--- NEWS	2006-04-04 00:28:28 +0000
+++ NEWS	2006-04-12 04:57:34 +0000
@@ -67,6 +67,10 @@
     * New 'init-repository' command, plus support for repositories in 'init'
       and 'branch' (Aaron Bentley, Erik Bågfors)
 
+    * 'pull' and 'push' now normalise the revision history, so that any two
+      branches with the same tip revision will have the same output from 'log'.
+      (Robert Collins)
+
   CHANGES:
 
     * Commit is now verbose by default, and shows changed filenames and the 

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-03-29 15:07:34 +0000
+++ bzrlib/branch.py	2006-04-12 04:57:34 +0000
@@ -333,9 +333,6 @@
         """
         raise NotImplementedError('update_revisions is abstract')
 
-    def pullable_revisions(self, other, stop_revision):
-        raise NotImplementedError('pullable_revisions is abstract')
-        
     def revision_id_to_revno(self, revision_id):
         """Given a revision id, return its revno"""
         if revision_id is None:
@@ -1016,20 +1013,49 @@
         # transaction.register_clean(history, precious=True)
         return list(history)
 
+    @needs_write_lock
     def update_revisions(self, other, stop_revision=None):
         """See Branch.update_revisions."""
-        if stop_revision is None:
-            stop_revision = other.last_revision()
-        ### Should this be checking is_ancestor instead of revision_history?
-        if (stop_revision is not None and 
-            stop_revision in self.revision_history()):
-            return
-        self.fetch(other, stop_revision)
-        pullable_revs = self.pullable_revisions(other, stop_revision)
-        if len(pullable_revs) > 0:
-            self.append_revision(*pullable_revs)
+        other.lock_read()
+        try:
+            if stop_revision is None:
+                stop_revision = other.last_revision()
+                if stop_revision is None:
+                    # if there are no commits, we're done.
+                    return
+            # whats the current last revision, before we fetch [and change it
+            # possibly]
+            last_rev = self.last_revision()
+            # we fetch here regardless of whether we need to so that we pickup
+            # filled in ghosts.
+            self.fetch(other, stop_revision)
+            my_ancestry = self.repository.get_ancestry(last_rev)
+            if stop_revision in my_ancestry:
+                # last_revision is a descendant of stop_revision
+                return
+            # stop_revision must be a descendant of last_revision
+            stop_graph = self.repository.get_revision_graph(stop_revision)
+            if last_rev is not None and last_rev not in stop_graph:
+                # our previous tip is not merged into stop_revision
+                raise errors.DivergedBranches(self, other)
+            # make a new revision history from the graph
+            current_rev_id = stop_revision
+            new_history = []
+            while current_rev_id not in (None, NULL_REVISION):
+                new_history.append(current_rev_id)
+                current_rev_id_parents = stop_graph[current_rev_id]
+                try:
+                    current_rev_id = current_rev_id_parents[0]
+                except IndexError:
+                    current_rev_id = None
+            new_history.reverse()
+            self.set_revision_history(new_history)
+        finally:
+            other.unlock()
 
+    @deprecated_method(zero_eight)
     def pullable_revisions(self, other, stop_revision):
+        """Please use bzrlib.missing instead."""
         other_revno = other.revision_id_to_revno(stop_revision)
         try:
             return self.missing_revisions(other, other_revno)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-04-09 06:19:54 +0000
+++ bzrlib/builtins.py	2006-04-12 04:57:34 +0000
@@ -360,15 +360,15 @@
             
     
 class cmd_pull(Command):
-    """Pull any changes from another branch into the current one.
+    """Turn this branch into a mirror of another branch.
 
     If there is no default location set, the first pull will set it.  After
     that, you can omit the location to use the default.  To change the
     default, use --remember.
 
     This command only works on branches that have not diverged.  Branches are
-    considered diverged if both branches have had commits without first
-    pulling from the other.
+    considered diverged if the destination branch's most recent commit is one
+    that has not been merged (directly or indirectly) into the parent.
 
     If branches have diverged, you can use 'bzr merge' to pull the text changes
     from one into the other.  Once one branch has merged, the other should
@@ -424,27 +424,28 @@
 
 
 class cmd_push(Command):
-    """Push this branch into another branch.
-    
-    The remote branch will not have its working tree populated because this
-    is both expensive, and may not be supported on the remote file system.
-    
-    Some smart servers or protocols *may* put the working tree in place.
+    """Update a mirror of this branch.
+    
+    The target branch will not have its working tree populated because this
+    is both expensive, and is not supported on remote file systems.
+    
+    Some smart servers or protocols *may* put the working tree in place in
+    the future.
 
     If there is no default push location set, the first push will set it.
     After that, you can omit the location to use the default.  To change the
     default, use --remember.
 
     This command only works on branches that have not diverged.  Branches are
-    considered diverged if the branch being pushed to is not an older version
-    of this branch.
+    considered diverged if the destination branch's most recent commit is one
+    that has not been merged (directly or indirectly) by the source branch.
 
     If branches have diverged, you can use 'bzr push --overwrite' to replace
-    the other branch completely.
+    the other branch completely, discarding its unmerged changes.
     
     If you want to ensure you have the different changes in the other branch,
-    do a merge (see bzr help merge) from the other branch, and commit that
-    before doing a 'push --overwrite'.
+    do a merge (see bzr help merge) from the other branch, and commit that.
+    After that you will be able to do a push without '--overwrite'.
     """
     takes_options = ['remember', 'overwrite', 
                      Option('create-prefix', 

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-04-05 01:02:06 +0000
+++ bzrlib/errors.py	2006-04-12 04:57:34 +0000
@@ -394,6 +394,7 @@
 
 
 class DivergedBranches(BzrError):
+
     def __init__(self, branch1, branch2):
         BzrError.__init__(self, "These branches have diverged.  Try merge.")
         self.branch1 = branch1

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2006-03-22 16:26:09 +0000
+++ bzrlib/revision.py	2006-04-12 04:57:34 +0000
@@ -23,6 +23,7 @@
 from bzrlib.graph import node_distances, select_farthest, all_descendants, Graph
 from bzrlib.osutils import contains_whitespace
 from bzrlib.progress import DummyProgress
+from bzrlib.symbol_versioning import *
 
 NULL_REVISION="null:"
 
@@ -400,6 +401,7 @@
             source.unlock()
 
 
+@deprecated_method(zero_eight)
 def get_intervening_revisions(ancestor_id, rev_id, rev_source, 
                               revision_history=None):
     """Find the longest line of descent from maybe_ancestor to revision.

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-04-05 00:59:14 +0000
+++ bzrlib/tests/__init__.py	2006-04-12 04:57:34 +0000
@@ -48,10 +48,12 @@
 import bzrlib.inventory
 import bzrlib.iterablefile
 import bzrlib.lockdir
+from bzrlib.merge import merge_inner
 import bzrlib.merge3
 import bzrlib.osutils
 import bzrlib.osutils as osutils
 import bzrlib.plugin
+from bzrlib.revision import common_ancestor
 import bzrlib.store
 import bzrlib.trace
 from bzrlib.transport import urlescape, get_transport
@@ -549,6 +551,22 @@
             sys.stderr = real_stderr
             sys.stdin = real_stdin
 
+    def merge(self, branch_from, wt_to):
+        """A helper for tests to do a ui-less merge.
+
+        This should move to the main library when someone has time to integrate
+        it in.
+        """
+        # minimal ui-less merge.
+        wt_to.branch.fetch(branch_from)
+        base_rev = common_ancestor(branch_from.last_revision(),
+                                   wt_to.branch.last_revision(),
+                                   wt_to.branch.repository)
+        merge_inner(wt_to.branch, branch_from.basis_tree(), 
+                    wt_to.branch.repository.revision_tree(base_rev),
+                    this_tree=wt_to)
+        wt_to.add_pending_merge(branch_from.last_revision())
+
 
 BzrTestBase = TestCase
 

=== modified file 'bzrlib/tests/branch_implementations/__init__.py'
--- bzrlib/tests/branch_implementations/__init__.py	2006-03-05 00:19:31 +0000
+++ bzrlib/tests/branch_implementations/__init__.py	2006-04-12 04:57:34 +0000
@@ -43,6 +43,7 @@
         'bzrlib.tests.branch_implementations.test_branch',
         'bzrlib.tests.branch_implementations.test_parent',
         'bzrlib.tests.branch_implementations.test_permissions',
+        'bzrlib.tests.branch_implementations.test_pull',
         'bzrlib.tests.branch_implementations.test_update',
         ]
     adapter = BranchTestProviderAdapter(

=== modified file 'bzrlib/tests/branch_implementations/test_bound_sftp.py'
--- bzrlib/tests/branch_implementations/test_bound_sftp.py	2006-03-04 13:03:57 +0000
+++ bzrlib/tests/branch_implementations/test_bound_sftp.py	2006-04-12 04:57:34 +0000
@@ -226,9 +226,9 @@
         # by pushing the new change to base
         wt_child.branch.bind(sftp_b_base)
 
-        self.assertEqual(['r@b-1', 'r@b-2', 'r@c-3'],
+        self.assertEqual(['r@b-1', 'r@c-2', 'r@c-3'],
                 b_base.revision_history())
-        self.assertEqual(['r@b-1', 'r@b-2', 'r@c-3'],
+        self.assertEqual(['r@b-1', 'r@c-2', 'r@c-3'],
                 wt_child.branch.revision_history())
 
     def test_bind_parent_ahead(self):

=== added file 'bzrlib/tests/branch_implementations/test_pull.py'
--- bzrlib/tests/branch_implementations/test_pull.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_pull.py	2006-04-12 04:57:34 +0000
@@ -0,0 +1,55 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for branch.pull behaviour."""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.osutils import abspath, realpath
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestPull(TestCaseWithTransport):
+
+    def test_pull_convergence_simple(self):
+        # when revisions are pulled, the left-most accessible parents must 
+        # become the revision-history.
+        parent = self.make_branch_and_tree('parent')
+        parent.commit('1st post', rev_id='P1', allow_pointless=True)
+        mine = parent.bzrdir.sprout('mine').open_workingtree()
+        mine.commit('my change', rev_id='M1', allow_pointless=True)
+        self.merge(mine.branch, parent)
+        parent.commit('merge my change', rev_id='P2')
+        mine.pull(parent.branch)
+        self.assertEqual(['P1', 'P2'], mine.branch.revision_history())
+
+    def test_pull_merged_indirect(self):
+        # it should be possible to do a pull from one branch into another
+        # when the tip of the target was merged into the source branch
+        # via a third branch - so its buried in the ancestry and is not
+        # directly accessible.
+        parent = self.make_branch_and_tree('parent')
+        parent.commit('1st post', rev_id='P1', allow_pointless=True)
+        mine = parent.bzrdir.sprout('mine').open_workingtree()
+        mine.commit('my change', rev_id='M1', allow_pointless=True)
+        other = parent.bzrdir.sprout('other').open_workingtree()
+        self.merge(mine.branch, other)
+        other.commit('merge my change', rev_id='O2')
+        self.merge(other.branch, parent)
+        parent.commit('merge other', rev_id='P2')
+        mine.pull(parent.branch)
+        self.assertEqual(['P1', 'P2'], mine.branch.revision_history())

=== modified file 'bzrlib/tests/repository_implementations/test_fileid_involved.py'
--- bzrlib/tests/repository_implementations/test_fileid_involved.py	2006-02-24 12:47:02 +0000
+++ bzrlib/tests/repository_implementations/test_fileid_involved.py	2006-04-12 04:57:34 +0000
@@ -19,8 +19,6 @@
 from bzrlib.add import smart_add
 from bzrlib.builtins import merge
 from bzrlib.delta import compare_trees
-from bzrlib.merge import merge_inner
-from bzrlib.revision import common_ancestor
 from bzrlib.tests.repository_implementations.test_repository import TestCaseWithRepository
 from bzrlib.workingtree import WorkingTree
 
@@ -32,17 +30,6 @@
         f.write("appended line\n")
         f.close( )
 
-    def merge(self, branch_from, wt_to):
-        # minimal ui-less merge.
-        wt_to.branch.fetch(branch_from)
-        base_rev = common_ancestor(branch_from.last_revision(),
-                                    wt_to.branch.last_revision(),
-                                    wt_to.branch.repository)
-        merge_inner(wt_to.branch, branch_from.basis_tree(), 
-                    wt_to.branch.repository.revision_tree(base_rev),
-                    this_tree=wt_to)
-        wt_to.add_pending_merge(branch_from.last_revision())
-
     def compare_tree_fileids(self, branch, old_rev, new_rev):
         old_tree = self.branch.repository.revision_tree(old_rev)
         new_tree = self.branch.repository.revision_tree(new_rev)

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2006-03-22 16:26:09 +0000
+++ bzrlib/tests/test_revision.py	2006-04-12 04:57:34 +0000
@@ -55,9 +55,8 @@
     tree1.commit("Commit two", rev_id="a@u-0-1")
     tree1.commit("Commit three", rev_id="a@u-0-2")
 
-    tree2 = self.make_branch_and_tree("branch2")
+    tree2 = tree1.bzrdir.clone("branch2").open_workingtree()
     br2 = tree2.branch
-    br2.update_revisions(br1)
     tree2.commit("Commit four", rev_id="b@u-0-3")
     tree2.commit("Commit five", rev_id="b@u-0-4")
     revisions_2 = br2.revision_history()

=== modified file 'tutorial.txt'
--- tutorial.txt	2006-01-30 04:37:01 +0000
+++ tutorial.txt	2006-04-12 04:57:34 +0000
@@ -106,9 +106,7 @@
 Branch locations
 ================
 
-All history is stored in a branch, which is just an on-disk directory containing control files.  There is no repository or database as used in svn or svk.
-
-  (There are proposals to add shared storage between related branches.)
+All history is stored in a branch, which is just an on-disk directory containing control files.  By default there is no separate repository or database as used in svn or svk. You can choose to create a repository if you want to (see the **bzr init-repo** command). You may wish to do this if you have very large branches, or many branches of a moderate sized project.
 
 You'll usually refer to branches on your computer's filesystem just by
 giving the name of the directory containing the branch.  bzr also supports
@@ -311,11 +309,14 @@
 
     % bzr pull
 
-After this change, the local directory will be a mirror of the source.
+After this change, the local directory will be a mirror of the source. This
+includes the ''revision-history'' - which is a list of the commits done in 
+this branch, rather than merged from other branches.
 
-This command only works if your local (destination) branch includes only
-changes from the parent branch and no commits of its own.  Otherwise, the
-branches are said to have ''diverged'', and they must be merged instead.
+This command only works if your local (destination) branch is either an
+older copy of the parent branch with no new commits of its own, or if the
+most recent commit in your local branch has been merged into the parent
+branch.
 
 Merging from related branches
 =============================

